; Helper procedure to read header
PRO Read_Header, $
  fid           , $  ; Input         
  n_Header_Lines, $  ; Input         
  Debug=Debug   , $  ; Input keyword 
  fCF=fCF       , $  ; Output keyword
  fLO=fLO            ; Output keyword

  ; Set up
  ; ...Set up error handler
  @osrf_pro_err_handler
  ; ...String search parameters
  FCF_STRING = "NGES CF"
  FLO_STRING = "LO freq"
  ; ...File variables
  Buffer = ''
  
  ; Loop over header lines
  FOR n = 1, n_Header_Lines DO BEGIN
    READF, fid, Buffer
    ; Extract bits from the header
    ; ...The central frequency
    IF ( STRCMP(Buffer,FCF_STRING,STRLEN(FCF_STRING)) ) THEN BEGIN
      fCF = DOUBLE((STRSPLIT(Buffer,":",/EXTRACT))[1])
    ENDIF
    ; ...The local oscillator frequency
    IF ( STRCMP(Buffer,FLO_STRING,STRLEN(FLO_STRING)) ) THEN BEGIN
      fLO = DOUBLE((STRSPLIT(Buffer,":",/EXTRACT))[1])
    ENDIF
  ENDFOR
  
END


;+
; oSRF method to load SRF data for a single AMSUA
; channel into an oSRF object.
; 
PRO OSRF::Load_AMSUA, $
  Sensor_Id  , $  ; Input
  Channel    , $  ; Input
  Path    = Path   , $ ; Input keyword
  Debug   = Debug  , $ ; Input keyword
  History = History    ; Output keyword
;-

  ; Set up
  ; ...OSRF parameters
  @osrf_parameters
  ; ...Set up error handler
  @osrf_pro_err_handler
  ; ...Check keywords
  Path = Valid_String(Path) ? Path : "."
  
  
  ; Parameters
  N_HEADER_LINES = 10
  HISTORY = "$Id: osrf__load_amsua.pro 20753 2012-08-24 18:40:01Z paul.vandelst@noaa.gov $"


  ; Get file name(s)
  ch = STRING(Channel,FORMAT='(i2.2)')
  ; ...Get list of *.inp files in path
  fileroot = Path+PATH_SEP()+Sensor_Id+"-"+ch+"*.inp"
  filelist = FILE_SEARCH(fileroot, COUNT = n_bands)
  ; ...Check result
  IF ( n_bands EQ 0 ) THEN $
    MESSAGE, 'Datafile(s) '+FILE_BASENAME(filelist) +' not found....', $
             NONAME=MsgSwitch, NOPRINT=MsgSwitch


  ; Allocate OSRF object
  ; ...Loop over bands to get number of SRF data points
  n_points = LONARR(n_bands)
  FOR i = 0, n_bands-1 DO BEGIN
    ; ...Make sure file exists
    fInfo = FILE_INFO(filelist[i])
    IF ( NOT fInfo.EXISTS ) THEN $
      MESSAGE, 'Datafile '+filelist[i]+' not found....', $
               NONAME=MsgSwitch, NOPRINT=MsgSwitch
    ; ...Count the lines
    n_points[i] = FILE_LINES(filelist[i]) - N_HEADER_LINES
  ENDFOR
  ; ...Perform the allocation
  self->Allocate, n_Points, Debug=Debug
  ; ...Clear all flags
  self->Clear_Flag, /All, Debug=Debug
  
  
  ; Loop over bands to read the data
  FOR i = 0, n_Bands-1 DO BEGIN
    band = i+1
    
    ; Read the current file
    OPENR, fid, filelist[i], /GET_LUN
    Read_Header, fid, N_HEADER_LINES, fLO = fLO
    x = DBLARR(4,n_points[i])
    READF, fid, x
    FREE_LUN, fid

    ; Create the frequency and response arrays
    f = MHz_to_GHz(fLO + REFORM(x[0,*]))
    r = dB_to_Response(REFORM(x[1,*]))
    ; ...Only keep the unique values
    idx = UNIQ(f, SORT(f))
    IF ( N_ELEMENTS(idx) NE n_points[i] ) THEN $
      MESSAGE, "Non-unique frequencies in band #"+STRTRIM(band,2), $
               NONAME=MsgSwitch, NOPRINT=MsgSwitch
    f = f[idx]
    r = r[idx]

    ; Load the SRF data into the oSRF object
    self->Set_Property, $
      Band, $
      Channel = Channel, $
      Frequency = f, $
      Response  = r, $
      f0 = f0
      
  ENDFOR  
  

  ; Done
  CATCH, /CANCEL
  
END ; PRO OSRF::Load_AMSUA
