;+
; Procedure to create oSRF netCDF files for AMSUA
;
PRO Write_AMSUA_SRF, $
  PS                 = PS                , $ ; Input keyword
  Path               = Path              , $ ; Input keyword
  No_Plot            = No_Plot           , $ ; Input keyword
  Version            = Version           , $ ; Input keyword
  SensorInfo_File    = SensorInfo_File   , $ ; Input keyword
  Debug              = Debug                 ; Input keyword
;-
                   
  ; Set up
  ; ...OSRF parameters
  @osrf_parameters
  ; ...Set up error handler
  @osrf_pro_err_handler
  ; ...Color definitions  
  @color_db
  ; ...Check keywords
  Path = Valid_String(Path) ? Path : "./"
  SensorInfo_File = Valid_String(SensorInfo_File) ? SensorInfo_File : "SensorInfo"
  Plot_Data = KEYWORD_SET(No_Plot) ? FALSE : TRUE
  Version   = (N_ELEMENTS(Version) GT 0) ? Version : OSRF_VERSION
  ; ...Plotting
  font  = KEYWORD_SET(ps) ? 1 : -1
  thick = KEYWORD_SET(ps) ? 2 :  1
  
  
  ; Parameters
  COMMENT_FILE = Path+PATH_SEP()+'source.comment'
  HISTORY = "$Id: write_amsua_srf.pro 20753 2012-08-24 18:40:01Z paul.vandelst@noaa.gov $"
  

  ; Get list of *.inp files in path
  filelist = FILE_BASENAME(FILE_SEARCH(Path+PATH_SEP()+"*.inp", COUNT = n_files))
  ; ...Extract platform from file
  platform = STRMID(filelist[0],STRPOS(filelist[0],"_")+1,3)


  ; Create the Sensor Id
  ; ...Determine if SensorInfo file is available
  finfo = FILE_INFO(SensorInfo_File)
  IF ( NOT finfo.EXISTS ) THEN $
    MESSAGE, "SensorInfo file "+SensorInfo_File+" not found.", $
             NONAME=MsgSwitch, NOPRINT=MsgSwitch
  ; ...Read the SensorInfo list
  sinfo_list = OBJ_NEW("SensorInfo_List", SensorInfo_File, Debug = Debug)
  sinfo_list->Read, Debug = Debug
  ; ...Construct the sensor id
  sensor_id = "amsua_" + platform
  MESSAGE, "Processing " + sensor_id + " SRF data...", /INFORMATIONAL
  ; ...Retrieve the sensorinfo information
  sinfo = sinfo_list->Get(Sensor_Id = sensor_id, Debug = Debug) 
  sinfo->Get_Property, Sensor_Channel   = sensor_channel  , $
                       WMO_Satellite_Id = wmo_satellite_id, $
                       WMO_Sensor_Id    = wmo_sensor_id   , $
                       Sensor_Type      = sensor_type     
  n_channels = N_ELEMENTS(sensor_channel)
  ; ...Cleanup
  OBJ_DESTROY, [sinfo, sinfo_list], Debug = Debug


  ; Output file global attributes
  ; ...The dataset title
  title = sensor_id+" Spectral Response Functions"
  ; ...The data source comment
  finfo = FILE_INFO(COMMENT_FILE)
  IF ( NOT finfo.EXISTS ) THEN $
    MESSAGE, "Comment file "+COMMENT_FILE+" not found.", $
             NONAME=MsgSwitch, NOPRINT=MsgSwitch
  OPENR, fid, COMMENT_FILE, /GET_LUN
  comment = ""
  READF, fid, comment
  FREE_LUN, fid

  
  ; Create the oSRF file container
  osrf_file = OBJ_NEW( 'OSRF_FILE', $
                       Path+PATH_SEP()+sensor_id + '.osrf.nc', $
                       Debug = Debug, $
                       Version = Version, $
                       Sensor_Id        = sensor_id       , $
                       WMO_Satellite_Id = wmo_satellite_id, $
                       WMO_Sensor_Id    = wmo_sensor_id   , $
                       Sensor_Type      = sensor_type     , $
                       Title   = title  , $
                       Comment = comment  )

 
  ; Open the PostScript file if required
  IF ( Plot_Data AND KEYWORD_SET(ps) ) THEN BEGIN
    pson, Filename = sensor_id+'.osrf.ps'
  ENDIF

  
  ; Begin the channel processing loop
  FOR l = 0, n_Channels-1L DO BEGIN
  
    ; Create a new oSRF object
    osrf = OBJ_NEW('OSRF', Debug = Debug)
    
    ; Load SRF data into object
    osrf->Load_AMSUA, sensor_id        , $
                      sensor_channel[l], $
                      Path    = Path   , $
                      Debug   = Debug  , $
                      History = load_history                
  
    ; Set the oSRF sensor ...
    ; ...properties
    osrf->Set_Property, Debug = Debug, $
                        Version = Version, $
                        Sensor_Id        = sensor_id       , $
                        WMO_Satellite_Id = wmo_satellite_id, $
                        WMO_Sensor_Id    = wmo_sensor_id   , $
                        Sensor_Type      = sensor_type     , $
                        Channel          = sensor_channel[l]
    ; ...and flags
    osrf->Set_Flag, /Frequency_Units


    ; Process the data
    osrf->Integrate, Debug = Debug
    osrf->Compute_Central_Frequency, Debug = Debug
    osrf->Compute_Planck_Coefficients, Debug = Debug 
    osrf->Compute_Polychromatic_Coefficients, Debug = Debug
    
    ; Add the oSRF to the file container
    osrf_file->Add, osrf, Debug = Debug

    ; Set the oSRF file container history
    IF ( l EQ 0 ) THEN $
      osrf_file->Set_Property, Debug = Debug, $
                               History = load_history + "; " + HISTORY

    ; Plot the data if required
    IF ( Plot_Data ) THEN BEGIN
      oSRF->Plot, TITLE = sensor_id + " channel " + STRTRIM(sensor_channel[l],2), $
                  FONT = font, $
                  THICK = thick
      IF ( NOT KEYWORD_SET(ps) ) THEN BEGIN
        q = GET_KBRD(1)
        IF ( STRUPCASE(q) EQ 'Q' ) THEN RETURN
      ENDIF
    ENDIF
    
  ENDFOR


  ; Close the PostScript file if required
  IF ( Plot_Data AND KEYWORD_SET(ps) ) THEN psoff
  
  ; Inspect the OSRF_File object for grins
  osrf_file->Inspect
 
  ; Write the OSRF_File object to file
  osrf_file->Write, Debug = Debug
  
  ; Cleanup
  OBJ_DESTROY, osrf_file, Debug = Debug
  
END ; PRO write_amsua_srf
